package com.simuwang.daq.components.writer;

import cn.hutool.core.exceptions.ExceptionUtil;
import com.simuwang.base.mapper.report.ReportBaseInfoMapper;
import com.simuwang.base.mapper.report.ReportFundInfoMapper;
import com.simuwang.base.pojo.dos.report.ReportBaseInfoDO;
import com.simuwang.base.pojo.dos.report.ReportFundInfoDO;
import com.simuwang.base.pojo.dto.report.ReportBaseInfoDTO;
import com.simuwang.base.pojo.dto.report.ReportData;
import com.simuwang.base.pojo.dto.report.ReportFundInfoDTO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StopWatch;

public abstract class AbstractReportWriter<T extends ReportData> implements ReportWriter<T> {
    private final Logger logger = LoggerFactory.getLogger(this.getClass());

    private final ReportBaseInfoMapper baseInfoMapper;
    private final ReportFundInfoMapper fundInfoMapper;

    public AbstractReportWriter(ReportBaseInfoMapper baseInfoMapper, ReportFundInfoMapper fundInfoMapper) {
        this.baseInfoMapper = baseInfoMapper;
        this.fundInfoMapper = fundInfoMapper;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void write(T reportData) {
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();
        try {
            // 基本信息+基金信息保存
            ReportBaseInfoDTO baseInfo = reportData.getBaseInfo();
            if (baseInfo != null) {
                ReportBaseInfoDO entity = baseInfo.toEntity();
                this.baseInfoMapper.insert(entity);
            }
            ReportFundInfoDTO fundInfo = reportData.getFundInfo();
            if (fundInfo != null) {
                ReportFundInfoDO entity = fundInfo.toEntity();
                this.fundInfoMapper.insert(entity);
            }
            try {
                // 其他信息保存
                this.writeExtData(reportData);
            } catch (Exception e) {
                this.logger.warn("报告解析结果之类型特有数据保存报错\n{}", ExceptionUtil.stacktraceToString(e));
            }
        } catch (Exception e) {
            this.logger.error("报告解析结果保存错误\n{}", ExceptionUtil.stacktraceToString(e));
        } finally {
            stopWatch.stop();
            long totalTimeMillis = stopWatch.getTotalTimeMillis();
            if (this.logger.isInfoEnabled()) {
                this.logger.info("报告解析结果保存成功，耗时：{}ms", totalTimeMillis);
            }
        }
    }

    protected abstract void writeExtData(T reportData);
}
